import * as React from 'react';
import { MutableRefObject } from 'react';
import { ForceConfig, ForceSimulation, SimulationGroup, SimulationLink, SimulationNode } from '@graph-viz/layouts';
import { ConfigurationOptions, DisplayGroup, DisplayLink, DisplayNode, GraphVisualization, VisualizationInputData } from '@graph-viz/core';
import { TooltipNode } from './NodeTooltips';
/**
 * Primary GraphVizData type definitions
 */
export declare type GraphVizNode = Partial<DisplayNode & SimulationNode> & Pick<DisplayNode, 'id'>;
export declare type GraphVizLink = DisplayLink & SimulationLink;
export declare type GraphVizGroup = DisplayGroup & SimulationGroup;
interface State {
    readonly currentTooltipNode: TooltipNode | null;
    readonly currentlyHoveredIdx: number | null;
    readonly draftLinkSourceNode?: GraphVizNode;
    readonly selectedNodes: GraphVizNode[];
}
export interface GraphVizComponentProps {
    /**
     * list of nodes to draw
     */
    nodes: GraphVizNode[];
    /**
     * list of links to draw
     */
    links: GraphVizLink[];
    /**
     * list of groups to draw
     */
    groups: GraphVizGroup[];
    /**
     * list of tooltips for nodes
     */
    tooltips: Partial<TooltipNode>[];
    /**
     * renders a refresh, zoom in and zoom out buttons if enabled
     */
    showControls?: boolean;
    /**
     * callback function when refresh button is clicked
     */
    onRefresh?: () => any;
    /**
     * callback function to handle errors
     * @param error
     */
    onError?: (error: Error) => any;
    /**
     * Graph config.
     * Defaults for nodes, links, and  groups
     */
    config?: ConfigurationOptions;
    /**
     * Force config.
     * Defaults for forces / layout computation
     */
    forceConfig?: ForceConfig;
    /**
     * dragMode selects what happens when you drag your cursor on the canvas
     * 'drag' drags a node or the entire graph
     * 'select' draws a rectangle and returns the list of selected nodes under
     * the rectangle
     * @see MouseInteraction.dragMode
     * @default 'drag'
     */
    dragMode: 'drag' | 'select';
    /**
     * enables graph editing
     * i.e. for now only the drawing of links
     */
    editMode?: boolean;
    /**
     * callback dispatched when a valid link is drawn.
     * requires editMode to be true
     */
    onLinkDrawn: (source: GraphVizNode, target: GraphVizNode) => any;
    /**
     * on mount this ref points to a function to trigger graph viz initialization
     */
    updateLayoutAndStylesRef?: MutableRefObject<() => any>;
    /**
     * on mount this ref points to a function to trigger graph viz updateStyles
     */
    updateStylesRef?: MutableRefObject<() => any>;
    /**
     * init callback called with the instances of GraphViz and ForceSimulation
     */
    onInit?: (viz: GraphVisualization, simulation: ForceSimulation) => any;
    /**
     * flag to prevent re-rendering the viz by default
     * useful in situation when you want to call render functions on the
     * GraphVisualization instance passed to the parent via onInit
     */
    preventAutomaticVizUpdate?: boolean;
}
export declare class GraphVizComponent extends React.Component<GraphVizComponentProps, State> {
    visualization: GraphVisualization;
    vizData: VisualizationInputData;
    tooltipNodes: TooltipNode[];
    simulation: ForceSimulation;
    rootRef: React.RefObject<HTMLDivElement>;
    canvasRef: React.RefObject<HTMLCanvasElement>;
    readonly state: State;
    static defaultProps: Partial<GraphVizComponentProps>;
    onWindowResize: () => void;
    componentDidMount(): void;
    componentDidUpdate(prevProps: GraphVizComponentProps): void;
    componentWillUnmount(): void;
    updateLayoutAndStyles: () => void;
    updateStyles: () => void;
    zoomIn: () => void;
    zoomOut: () => void;
    render(): JSX.Element;
}
export {};
//# sourceMappingURL=GraphVizComponent.d.ts.map